%% Allocation of balancing reserves considering V2G technology and energy Bids.
% Verified: 24/06

function [AS_up, AS_down, Ene_AS_down, Ene_AS_up, SOC_AS_out, Max_down_P, Max_up_P] =...
    AS_Energy_Basic (SOC_0_AS, Bid, P_up, P_down, EV_batt, EV_n, EV_P, ...
    ConsumE, available, value_freq, day, conv_top)

%% Local variables
Conn_EV = available * EV_n; % number of EVs connected to the grid
Max_SOC = (EV_batt * EV_n)/1000; % MWh
hl = length(value_freq)/6; % represents the length of four hour block of frequency data
Pr_limit = hl/16; % number of straight frequency measurements that can provide primary reserve (15 minutes)

%% Initialize variables
AS_down = zeros(1,6); % By default the blocks provide regulation
AS_up = zeros(1,6);
freq_db = 0.02; % Hz - frequency deadband for A/S response. Corresponds to x0 of the curve
Max_freq = 0.2; % Hz - maximum frequency range for BRP to respond balancing deviations
ctr_freq_down = 0;
ctr_freq_up = 0;
freq_pr_down = zeros(1,length(value_freq)); 
freq_pr_up = zeros(1,length(value_freq));
Ene_AS_up = zeros(1,length(Bid));
Ene_AS_down = zeros(1,length(Bid));
SOC_AS_in = cumsum(cat(2,SOC_0_AS,(Bid-ConsumE))); % Initialize variable. It will be overwriten
SOC_AS_out = SOC_AS_in;
bndex = 0;
Max_down_P = zeros (1,6);
Max_up_P = zeros (1,6);

%% Select frequency for primary reserves.
% It has to be within a certain range and cannot last for more than 15
% minutes.
f = 0;
    while f<length(value_freq)
        f = f + 1;
        if value_freq (f) > 50.02 & value_freq (f) < 50.2
            freq_pr_down (f) = value_freq (f);
            ctr_freq_down = ctr_freq_down + 1; 
        else
            ctr_freq_down = 0;
        end
        if ctr_freq_down == Pr_limit
            f = f + Pr_limit;
            ctr_freq_down = 0; % reset the counter one the limit has been reached
        end
    end

f = 0;
    while f<length(value_freq)
        f = f + 1;
        if value_freq (f) < 49.98 & value_freq (f) > 49.8
            freq_pr_up (f) = value_freq (f);
            ctr_freq_up = ctr_freq_up + 1;
        else
            ctr_freq_up = 0;
        end
        if ctr_freq_up == Pr_limit
            f = f + Pr_limit;
            ctr_freq_down = 0; % reset the counter one the limit has been reached
        end
    end

%% Loop to calculate which block will provide which A/S
for b = 3:4:24
    bndex = bndex + 1;
    if SOC_AS_in (b) < 0.9 * Max_SOC
        % Provide downward regulation in block i
        AS_down (bndex) = 1;
    end
    if SOC_AS_in (b) > 0.3 * Max_SOC
        % Provide upward regulation in block i
        AS_up (bndex) = 1;
    end
end
    
    %% Calculating power absorbed by down regulation
for i = 1:6
    if AS_down (i) == 1 % access block level
        ini_b = (hl * (i-1))+1; % Initial value of the block
        fin_b = hl * i; % Final value of the block
        freq_blk = freq_pr_down(ini_b:fin_b); % arrange frequency in the correspondent block
        Max_down_P (i) = min(Conn_EV((i*4-3):(i*4))* EV_P/1000 - Bid((i*4-3):(i*4))); % MW - maximum power available to provide in downward regulation (consume) for a specific block
            if Max_down_P (i) < 0.3 % Minimum size for bidding is 300 kW
                fprintf('Downward regulation bid cannot be made for block %d in day %d due to insuficient power \n',i,day)
                Ene_AS_down ((i-1)*4+1:i*4) = 0;
                continue
            end
        m_down = Max_down_P (i)/ (Max_freq - freq_db); % MW/Hz - slope of the curve     
        for j = 1:4 % access each hour inside previous block
            ini_h = ((hl/4) * (j-1))+1; % Initial value of hour within the block
            fin_h = (hl/4) * j;
            freq_pr_h = freq_blk(ini_h:fin_h); % arrange frequency in the correspondent hour
                for f = 1:length(freq_pr_h) % compares all the frequencies in the selected hour
                    if freq_pr_h(f) > 0
                        freq_dev = abs(50.02-freq_pr_h(f));
                        P(f) = freq_dev * m_down; % Power applied is proportional to frequency deviation.
                    else
                        P(f) = 0;
                    end
                end
             Ene_AS_down ((i-1)*4+j) = sum(P)/(hl/4); % Total energy charged in the block
        end
    else
        Ene_AS_down ((i-1)*4+1:i*4) = 0;
        fprintf('Downward regulation bid cannot be made for block %d in day % d due to high SOC \n',i,day)
    end  
end
clearvars P

%% Calculating power absorved by up regulation (V2G)
for i = 1:6
    if AS_up (i) == 1 % access block level
        ini_b = (hl * (i-1))+1;
        fin_b = hl * i;
        freq_blk = freq_pr_up(ini_b:fin_b); % arrange frequency in the correspondent block
        if conv_top == 1 
            Max_up_P (i)= min(Bid((i*4-3):(i*4))); % MW - maximum power available to provide in downward regulation (consume) for a specific block
        else
            Max_up_P (i)= min(Conn_EV((i*4-3):(i*4))* EV_P/1000 + Bid((i*4-3):(i*4)));         
        end
            if Max_up_P (i)< 0.3 % Minimum size for bidding is 300 kW
                fprintf('Upward regulation bid cannot be made for block %d in day %d due to insuficient power\n',i,day)
                Ene_AS_up ((i-1)*4+1:i*4) = 0;
                continue
            end
        m_up = Max_up_P (i)/ (Max_freq - freq_db); % MW/Hz - slope of the curve        
        for j = 1:4 % access each hour inside previous block
            ini_h = ((hl/4) * (j-1))+1;
            fin_h = (hl/4) * j;
            freq_pr_h = freq_blk(ini_h:fin_h); % arrange frequency in the correspondent block
                for f = 1:length(freq_pr_h) % compares all the frequencies in the selected hour
                    if freq_pr_h(f) > 0
                        freq_dev = abs(49.98-freq_pr_h(f));
                        P(f) = freq_dev * m_up; 
                    else
                        P(f) = 0;
                    end
                end
             Ene_AS_up ((i-1)*4+j) = sum(P)/(hl/4); % Total energy charge in the block
        end
    else
        Ene_AS_up ((i-1)*4+1:i*4) = 0;
        fprintf('Upward regulation bid cannot be made for block %d in day %d due to low SOC \n',i,day)
    end  
end

for j = 1 : length(Ene_AS_down)
    SOC_AS_out (j+1) = SOC_AS_in(j+1) + sum (Ene_AS_down(1:j)) - sum(Ene_AS_up(1:j));
    if SOC_AS_out(j+1) < 0.2 * Max_SOC
        A = int32(j);
        C = idivide(A,4,'ceil');
        fprintf('Lower level of SOC reached for block %d in day %d. No upward regulation accepted in this block\n',C,day)
        for k = j:4*C
            Ene_AS_up(k) = 0;
        end
    elseif SOC_AS_out(j+1) > 0.95 * Max_SOC
        A = int32(j);
        C = idivide(A,4,'ceil');
        fprintf('Higher level of SOC reached for block %d in day %d. No downward regulation accepted in this block\n',C,day)
        for k = j:4*C
            Ene_AS_down(k) = 0;
        end
    end
end
