%% Function to calculate energy that can be used for adaptive charging

function[SOC, Bid, P_upward, P_downward, ConsumE, fval_up, fval_down] = ...
    Adapt_Basic (EV_n, EV_P, EV_E, EV_batt, subrange, range_1, ...
    range_2, Penetration,usage,available,charging,SOC_0,day,conv_topo,scheme)


%% Initialize variables
SOC = zeros(1,length(subrange)+1);
f_P = zeros(1,length(subrange)); % Penetration factor
f_SOC = zeros(1,length(subrange)); % SOC factor
Bid = zeros(1,length(subrange)); % Power to be bid in day ahead market
P_upward = 0;
P_downward = 0;
fval_up = 0;
fval_down = 0;

%% Calculated variables
driven_EV = usage * EV_n; % number of EVs being driven
Avai_P_EV = available*EV_n*EV_P/1000; % MW - available power to be absorved by EV at every hour of the day
Max_SOC = (EV_batt * EV_n)/1000; % MWh
ConsumE = (driven_EV * EV_E)/1000; % MWh - Energy consumed by EV when being driven
SOC (1) = SOC_0; % MW - SOC when the stated hour starts.

%% Loop to calculate bid
counter = 0;
for p=1:24
    if Penetration(p)<0.5
        counter=counter+1;
    end
end

for i = 1:length(range_1) % belonging to range 1
    if counter >= 17 & SOC(1) < 0.6 * Max_SOC
        f_P (3:6) = 1;
        break
    end
    index = range_1(i); 
    if Penetration(i) > 1
        f_P (index) = 1; %Avai_P_EV (index);
    elseif Penetration(index) > 0.75
        f_P (index) = 0.9; % * Avai_P_EV (index);
    elseif Penetration(index) > 0.5
        f_P (index) = 0.8; % * Avai_P_EV (index); % Only 50% of the available power is bided       
    else
        f_P (index) = 0;
    end
end

for i = 1:length(range_2) % belonging to range 2
    if counter >= 17 & SOC(1) < 0.6 * Max_SOC % No high penetration has already been foreseen
        break
    end
    index = range_2(i); 
    if Penetration(index) > 1
        f_P (index) = 0.8;   
    elseif Penetration(index) > 0.75
        f_P (index) = 0.5;
    elseif Penetration(index) > 0.5
        f_P (index) = 0.3;      
    else
        f_P (index) = 0;
    end
end

%% Loop considering SOC
for j = 1:length(subrange)
    if SOC(j) > 0.8*Max_SOC
        f_SOC(j) = 0;
    elseif SOC(j) > 0.7*Max_SOC
       f_SOC(j) = 0.2;
    elseif SOC(j) > 0.6*Max_SOC
        f_SOC (j)= 0.5;        
    elseif SOC(j) > 0.5*Max_SOC
        f_SOC (j)= 0.75;   
    elseif SOC(j) > 0.3*Max_SOC
        f_SOC (j)= 1;
    else
        f_P (j) = 1; % It charges all available energy
        f_SOC (j) = 1;
    end
        Bid (j)= f_P (j) * f_SOC (j) * available (j) * (0.95*Max_SOC-SOC(j));
    
    if Bid(j) > Avai_P_EV (j)
        Bid(j) = Avai_P_EV (j);
    end
    
    SOC (j+1) = SOC (j) + Bid(j) - ConsumE(j);
       
end
