function [SOC_adapt, Bid, P_upward, P_downward, ConsumE, fval_up,...
    fval_down] = Adapt_optimal_DE (EV_n, EV_P, EV_E, EV_batt, subrange,...
    range_1, range_2, Penet_local,usage_local,available_local,charging,...
    SOC_ini,day, conv_topo,scheme)

% Define variables        
Max_SOC = (EV_batt * EV_n)/1000; % MWh
Avai_P_EV = available_local*EV_n*EV_P/1000; % MW - available power fleet
ConsumE = (usage_local * EV_n* EV_E)/1000; % MWh - Energy consumed fleet
for i = 1:6 % Block variables
    Media(i) = mean(Penet_local(i*4-3:i*4));
    ConsumE_blk(i) = sum(ConsumE(i*4-3:i*4)); % Energy consumed
    min_Avai_P (i) = min(Avai_P_EV(i*4-3:i*4)); % minimum power available
end
f_AS_up = [0.004 0.006 0.007 0.008 0.021 0.04]; % Percentile 20th
f_AS_down = [0.049 0.041 0.040 0.035 0.059 0.112]; % Percentile 80th
SOC_max = [0.9 0.89 0.88 0.87 0.86 0.85] * Max_SOC;

% Initialize variables
D = 6;
XVmin = 0;
XVmax = 1;
NP = 10*D;
fevalmax = 18000;
F = 0.6;
CR = 0.9;
strategy = 3;
refresh = 0;

save variables_up.mat SOC_ini ConsumE_blk Media min_Avai_P SOC_max f_AS_up Max_SOC
fprintf('Day %.0f, calculating upward regulation \n',day)
%% Optimization function for upward provision of frequency regulation
if conv_topo == 1 % uni-directional
    fname = 'OF_up_DE_uni'; % objective function for upward regulation
    [bestmem,fval_up,nfeval,OFbest,bestvec] = ...
        DE_func_two(fname,D,XVmin,XVmax,NP,fevalmax,F,CR,strategy,refresh);
else % bi-directional
    fname = 'OF_up_DE_bi';
    [bestmem,fval_up,nfeval,OFbest,bestvec] = ...
        DE_func(fname,D,XVmin,XVmax,NP,fevalmax,F,CR,strategy,refresh);
end
    
% Outputs
P_upward = bestmem; % Bid Upward regulation and Bid day-ahead market
P_pre_down = min_Avai_P - P_upward;
for i = 1:6
    Bid(i*4-3:i*4) = repmat(P_upward(i),1,4);
end

SOC_up = [SOC_ini zeros(1,24)];
for i=1:24
    SOC_up(i+1) = SOC_up(i) + Bid(i)*(1-f_AS_up(ceil(i/4))) - ConsumE(i);
end

%% Optimization function for downward provision of frequency regulation
save variables_down.mat SOC_up P_pre_down Max_SOC f_AS_down
fprintf('Day %.0f, calculating Downward regulation \n',day)
if conv_topo == 1 % uni-directional
    fname = 'OF_down_DE'; % objective function for upward regulation
    [bestmem,fval_down,nfeval,OFbest,bestvec] = ...
        DE_func_two(fname,D,XVmin,XVmax,NP,fevalmax,F,CR,strategy,refresh);
else % bi-directional
    fname = 'OF_down_DE';
    [bestmem,fval_down,nfeval,OFbest,bestvec] = ...
        DE_func_two(fname,D,XVmin,XVmax,NP,fevalmax,F,CR,strategy,refresh);
end

% Outputs    
P_downward = bestmem; % Bring variable back to real value

SOC_adapt (1) = SOC_up(1);
for i=1:24
    SOC_adapt(i+1) = SOC_adapt(i) + Bid(i)*(1-f_AS_up(ceil(i/4)))...
        - ConsumE(i) + P_downward(ceil(i/4))*f_AS_down(ceil(i/4));
end