%% This program studies Ancillary Services with unidirectional technology.
% Revised: 02/07
tic
clear all; clc; close all;
set(0,'DefaultAxesFontSize',16)
set(0,'DefaultLineLineWidth',1.5)

%% Variable Definition

EV_n = 400; % number of electric vehicles
EV_P = 5; % kW - power at connection for every EV. Cars will not be charged
EV_E = 3.2; % kWh - energy consumed per hour per EV, considering daily consumption of 8 kWh and whole daily trip completed in 2 hours
EV_batt = 30; %kWh - total storage capacity of battery
Max_SOC = (EV_batt * EV_n)/1000; % MWh
SOC_0 = Max_SOC * 0.6; % MWh - Initial SOC of batteries
SOC_0_AS = SOC_0; % MWh
cf_wind_20 = 1.32; % correction factor, indicates increment in wind production by 2020
cf_con_20 = 1.018; % correction factor for consumption, increment in consumption of 1,8% by 2020
f_AS_up = [0.02 0.03 0.02 0.03 0.05 0.07]; % average value
f_AS_down = [0.03 0.03 0.02 0.02 0.04 0.08]; % average value
fval_up_w = 0;
fval_down_w = 0;

% Extracting data
load 'DK1.mat'
W_prog_14 = DK1.four.wind.prognosis; % MW - wind prognosis
W_prog_20 = W_prog_14 * cf_wind_20; 
C_prog_14 = DK1.four.consumption.prognosis; % MW - Consumption prognosis
C_prog_20 = C_prog_14 * cf_con_20;
spot = DK1.four.spot; % DKK - prices in the Elspot market

Penetration_20 = (W_prog_20)./C_prog_20;

load EV_usage.mat % structure defining EV patterns
EV_usage = usage.four;
EV_available = available.four;
EV_charging = charging.four;

load freq_DK1_60.mat % Each hour has 60 frequency measurements (every 1 minute)

load Day.mat % Predifine structure to store the data
campo = fields(Day);


flag_sch = 0;
flag_conv = 0;
flag_week = 0;
% Choose charging scheme
while flag_sch == 0
prompt = 'Choose the charging scheme: \n 1 - Basic \n 2 - GSA \n 3 - GA \n 4 - DE \n 5 - Non-controlled \n';
sch = input(prompt);
if sch == 1 | sch == 2 | sch == 3 | sch == 4 | sch == 5
    flag_sch = 1;
else
    disp('value not accepted. Choose again')
end
end

% Chose converter capability
if sch == 5
    conv = 1; tab_name_2 = [];
else
    while flag_conv == 0
        prompt = 'Choose the converter capability: \n 1 - Uni-directional \n 2 - Bi-directional \n ';
        conv = input(prompt);
        if conv == 1
            flag_conv = 1; tab_name_2 = '_uni';
        elseif conv == 2
            flag_conv = 1; tab_name_2 = '_bi';
        else
            disp('value not accepted. Choose again')
        end
    end
end

% Chose week
if sch == 5
    F_day = 244; L_day = 250; tab_name_3 = [];
else
    while flag_week == 0
        prompt = 'Choose level of penetration for the week: \n 1 - Low \n 2 - High \n ';
        week = input(prompt);
        if week == 1
            flag_week = 1; F_day = 244; L_day = 250; tab_name_3 = '_L';
        elseif week == 2
            flag_week = 1; F_day = 349; L_day = 355; tab_name_3 = '_H';
        else
            disp('value not accepted. Choose again')
        end
    end
end

switch sch
    
    case 1
        fname_1 = 'Adapt_Basic';... % Basic Adaptive scheme
            fname_2 = 'AS_Energy_Basic'; 
            tab_name_1 = 'Basic';
    case 2
        fname_1 = 'Adapt_optimal_GSA';... % Optimized scheme with Gradient Search Algorithm
            fname_2 = 'AS_Energy';
            tab_name_1 = 'GSA';
    case 3 
        fname_1 = 'Adapt_optimal_GA';... % Optimized scheme with Genetic Algorithm
            fname_2 = 'AS_Energy';
            tab_name_1 = 'GA';
    case 4
        fname_1 = 'Adapt_optimal_DE';... % Optimized scheme with Differential Evolution Algorithm
            fname_2 = 'AS_Energy';
            tab_name_1 = 'DE';
    case 5
        fname_1 = 'Non_Adapt';... % Basic Adaptive scheme
           fname_2 = 'AS_Energy';  
           tab_name_1 = 'Non_Adapt';
end

% Dividing by days
% Week with high penetration= 349:355
% Week with low penetration= 244:250
% F_day = 349;
% L_day = 355;
Range = F_day*24-23:L_day*24; % Amount of hours to perform the test
subrange = 1:24; % It represents the length of a day
range_1 = [1:7 23:24]; % Low demand hours
range_2 = 8:22; % High demand hours
day = 24;
m = length(Range)/24;
fh = length(value_freq)/(365*24); % Number of frequency measurements per hour
Penetration = Penetration_20(Range);

if L_day-F_day > 6
    disp('The range of days must be 7 or less')
    return
end

%% Loop for calling the functions

    for i = 1:m % separate each day
        disp('------------------------------------------------------------------\n')
        fprintf('Day %.0f \n SOC_0 = %.2f \n SOC_0_AS = %.2f \n',i,SOC_0,SOC_0_AS)        
        
        % Definition of local variables
        ini = Range((day * (i-1))+1);
        fin = Range(day * i);
        usage_local = EV_usage(ini:fin);
        charging_local = EV_charging(ini:fin); % only applicable to Non-controlled scheme
        available_local = EV_available(ini:fin);
        Penet_local = Penetration_20(ini:fin);
        freq_local = value_freq((F_day+i-2)*day*fh+1:(F_day+i-1)*day*fh);%(ini*fh:fin*fh);
        
        
        % Optimize function for Ancillary Services power allocation
        [SOC, Bid, P_up, P_down, ConsumE, fval_up, fval_down] = ... % SOC is estimated SOC
            feval(fname_1,EV_n, EV_P, EV_E, EV_batt, subrange, range_1,...
            range_2, Penet_local,usage_local,available_local, charging_local, SOC_0,i,conv,sch);
        

        [AS_up, AS_down, Ene_AS_down, Ene_AS_up, SOC_AS, P_down_AS, P_up_AS] = ...
            feval(fname_2,SOC_0_AS, Bid, P_up, P_down, EV_batt, EV_n, EV_P,... 
            ConsumE, available_local, freq_local, i, conv);
        
        if sch == 1
            P_down = P_down_AS;
            P_up = P_up_AS;
        end
        
        % Initialize for next day
        SOC_0 = SOC(25) - (SOC(13)-SOC_AS(13))...
            + sum(f_AS_down (4:6) .* P_down (4:6))...
            - sum(f_AS_up (4:6) .* P_up (4:6));
        SOC_0_AS = SOC_AS(25);
             
        %% Assigning  values to the structure
        Day.(campo{i}).SOC = SOC(1:end-1);
        Day.(campo{i}).SOC_AS = SOC_AS(1:end-1);
        Day.(campo{i}).Bid = Bid;
        Day.(campo{i}).AS_down = AS_down;
        Day.(campo{i}).AS_up = AS_up;
        Day.(campo{i}).Power_down_blk = P_down; 
        Day.(campo{i}).Power_up_blk = P_up; 
        Day.(campo{i}).Ene_AS_down = Ene_AS_down;
        Day.(campo{i}).Ene_AS_up = Ene_AS_up;
        
        fval_up_w = fval_up_w + fval_up;
        fval_down_w = fval_down_w + fval_down;
        
        if SOC_AS (25) < 0.2*Max_SOC
            disp('SOC lower limit reached')
            break
        elseif SOC_AS (25) > 0.95*Max_SOC
            disp('SOC higher limit reached')
            break
        end
        
        % Clear temporary variables
        clearvars SOC_non_adapt Charg_non_adapt SOC Bid AS_up AS_down...
            Ene_AS_down Ene_AS_up Cost_non Cost_adapt SOC_AS P_down P_up
        % Clear local variables
        clearvars ini fin usage_local Penet_local spot_local freq_local
        
    end
    
%% concatenate variables to plot
% IMPORTANT!! The SOC are the only vectors with 25 values. In order not to be repeated, in the SOC the last one is suppresed. 

SOC_cat = Day.one.SOC;
Bid_cat = Day.one.Bid;
AS_up_cat = Day.one.AS_up;
AS_down_cat = Day.one.AS_down;
Ene_AS_down_cat = Day.one.Ene_AS_down;
Ene_AS_up_cat = Day.one.Ene_AS_up;
SOC_AS_cat = Day.one.SOC_AS;
Power_down_blk_cat = Day.one.Power_down_blk;
Power_up_blk_cat = Day.one.Power_up_blk;

for j=2:i % Loop as many days has been computed.
    SOC_cat = cat(2,SOC_cat,Day.(campo{j}).SOC);
    Bid_cat = cat(2,Bid_cat,Day.(campo{j}).Bid);
    AS_up_cat = cat(2,AS_up_cat,Day.(campo{j}).AS_up);
    AS_down_cat = cat(2,AS_down_cat,Day.(campo{j}).AS_down);
    Ene_AS_down_cat = cat(2,Ene_AS_down_cat,Day.(campo{j}).Ene_AS_down);
    Ene_AS_up_cat = cat(2,Ene_AS_up_cat,Day.(campo{j}).Ene_AS_up);
    SOC_AS_cat = cat(2,SOC_AS_cat,Day.(campo{j}).SOC_AS);
    Power_down_blk_cat = cat(2,Power_down_blk_cat,Day.(campo{j}).Power_down_blk);
    Power_up_blk_cat = cat(2,Power_up_blk_cat,Day.(campo{j}).Power_up_blk);
end

%% Plots
% SOC with adaptive charging and A/S
plot_range = Range(1):Range(1)+length(SOC_AS_cat)-1;
figure(1)
plot(plot_range,SOC_AS_cat,'r',plot_range,SOC_cat)
title ('SOC Estimated and SOC measured after providing AS')
xlim([Range(1) Range(end)])
ylim([0 1.2*Max_SOC])
legend('SOC Ancillary Services','SOC Adaptive charging')
ylabel('SOC')
xlabel('Time in hours')
line('XData', [Range(1) Range(1)+length(SOC_AS_cat)], 'YData', [0.95*Max_SOC 0.95*Max_SOC], 'LineStyle', ':', 'Color','r')
line('XData', [Range(1) Range(1)+length(SOC_AS_cat)], 'YData', [0.2*Max_SOC 0.2*Max_SOC], 'LineStyle', ':', 'Color','r')
line('XData', [Range(1) Range(1)+length(SOC_AS_cat)], 'YData', [0.9*Max_SOC 0.9*Max_SOC], 'LineStyle', ':', 'Color','k')
line('XData', [Range(1) Range(1)+length(SOC_AS_cat)], 'YData', [0.3*Max_SOC 0.3*Max_SOC], 'LineStyle', ':', 'Color','k')

% SOC with adaptive charging and bids placed
figure(2)
[ax_1,p1_1,p2_1] = plotyy(Range,SOC_AS_cat,Range,Bid_cat,'plot','bar');
title ('SOC of battery with adaptive charging and bids placed')
set(ax_1(1),'xlim',[Range(1) Range(end)]);
set(ax_1(1),'ylim',[0 Max_SOC]);
set(ax_1(2),'xlim',[Range(1) Range(end)]);
set(ax_1(2),'ylim',[0 1.5]);
ylabel(ax_1(1),'SOC')
ylabel(ax_1(2),'Power Bid')
xlabel('Time in hours') 
line('XData', [Range(1) Range(end)], 'YData', [0.2*Max_SOC 0.2*Max_SOC], 'LineStyle', '-.','Color','k')
line('XData', [Range(1) Range(end)], 'YData', [0.3*Max_SOC 0.3*Max_SOC], 'LineStyle', ':','Color','m')
line('XData', [Range(1) Range(end)], 'YData', [0.9*Max_SOC 0.9*Max_SOC], 'LineStyle', ':','Color','m')
for l = 1:m-1
    line('XData', [Range(24*l+1) Range(24*l+1)], 'YData', [0 Max_SOC], 'LineStyle', ':')
end

% 2020 Wind Penetration
figure(3)
plot(Range,Penetration_20(Range))
title ('Level of wind penetration')
xlim([Range(1) Range(end)])
ylim([0 1.2*max(Penetration_20(Range))])
ylabel('Penetration')
xlabel('Time in hours')
line('XData', [Range(1) Range(end)], 'YData', [0.5 0.5], 'LineStyle', ':', 'Color','b')
line('XData', [Range(1) Range(end)], 'YData', [0.75 0.75], 'LineStyle', ':', 'Color','c')
line('XData', [Range(1) Range(end)], 'YData', [1 1], 'LineStyle', ':', 'Color','g')

% Ancillary Services Energy
Total_up = sum(Ene_AS_up_cat);
Total_down = sum(Ene_AS_down_cat);
bla = sprintf('EVs = %.0f \n Energy Up %.2f MWh \n Energy Down %.2f MWh',EV_n,Total_up,Total_down);
figure(4)
bar(-1*Ene_AS_up_cat)
hold on
bar(Ene_AS_down_cat)
title('Energy flow to the battery by providing frequency regulation')
xlim([1 m*24])
ylim([-0.25 0.15])
for l = 1:m-1
    line('XData', [24*l+1 24*l+1], 'YData', [-0.25 0.15], 'LineStyle', ':')
end
annotation('textbox',[0.6,0.8,0.2,0.1], 'FitHeightToText','on','String', bla,'FontSize',12)

% Representation of the power bids to ancillary services.
figure (5)
bar(-1*Power_up_blk_cat)
hold on
bar(Power_down_blk_cat)
title('Power Bids to frequency regulation market')
xlim([1 m*6])
ylim([-2.5 2])
line('XData', [1 m*6], 'YData', [0.3 0.3], 'LineStyle', ':', 'Color','m')
line('XData', [1 m*6], 'YData', [-0.3 -0.3], 'LineStyle', ':','Color','m')
for l = 1:m-1
    line('XData', [6*l+1 6*l+1], 'YData', [-2.5 2], 'LineStyle', ':')
end

%% save values to .mat file   
filename = strcat('Tables\Week_Results\',tab_name_1,tab_name_2,tab_name_3,'.mat');    
save(filename,...
    'Power_up_blk_cat','Power_down_blk_cat','Penetration','Ene_AS_up_cat',...
    'Ene_AS_down_cat','SOC_cat','Bid_cat','fval_up_w','fval_down_w')
toc