%% This program studies Ancillary Service provision by using a differential evolution algorithm
% Revised: 06/07
tic
clear all; clc; close all;
set(0,'DefaultAxesFontSize',16)
set(0,'DefaultLineLineWidth',1.5)

%% Variable Definition

EV_n = 400; % number of electric vehicles
EV_P = 5; % kW - power at connection for every EV. Cars will not be charged
EV_E = 3.2; % kWh - energy consumed per hour per EV, considering daily consumption of 8 kWh and whole daily trip completed in 2 hours
EV_batt = 30; % kWh - total storage capacity of battery
Max_SOC = (EV_batt * EV_n)/1000; % MWh
SOC_0 = Max_SOC * 0.6; % MWh - Initial SOC of batteries
SOC_0_AS = SOC_0; % MWh
cf_wind_20 = 1.32; % correction factor, indicates increment in wind production by 2020
cf_con_20 = 1.018; % correction factor for consumption, increment in consumption of 1,8% by 2020
f_AS_up = [0.02 0.03 0.02 0.03 0.05 0.07];
f_AS_down = [0.03 0.03 0.02 0.02 0.04 0.08];

% Extracting data
load 'DK1.mat'
W_prog_14 = DK1.four.wind.prognosis; % MW - wind prognosis
W_prog_20 = W_prog_14 * cf_wind_20;
C_prog_14 = DK1.four.consumption.prognosis; % MW - Consumption prognosis
C_prog_20 = C_prog_14 * cf_con_20;
spot = DK1.four.spot; % DKK - prices in the Elspot market

Penetration_20 = (W_prog_20)./C_prog_20;

load EV_usage.mat
EV_usage = usage.four;
EV_available = available.four;
EV_charging = charging.four;

load freq_DK1_60.mat % Each hour has 60 frequency measurements (every 1 minute)

load Day.mat % Predifine structure to store the data
campo = fields(Day);

flag_sch = 0;
flag_conv = 0;
% Choose charging scheme
while flag_sch == 0
prompt = 'Choose the charging scheme: \n 1 - Basic \n 2 - GSA \n 3 - GA \n 4 - DE \n 5 - Non-controlled \n';
sch = input(prompt);
if sch == 1 | sch == 2 | sch == 3 | sch == 4 | sch == 5
    flag_sch = 1;
else
    disp('value not accepted. Choose again')
end
end

% Chose converter capability
if sch == 5
    conv = 1; tab_name_2 = [];
else
    while flag_conv == 0
        prompt = 'Choose the converter capability: \n 1 - Uni-directional \n 2 - Bi-directional \n ';
        conv = input(prompt);
        if conv == 1
            flag_conv = 1; tab_name_2 = '_uni';
        elseif conv == 2
            flag_conv = 1; tab_name_2 = '_bi';
        else
            disp('value not accepted. Choose again')
        end
    end
end

switch sch
    
    case 1
        fname_1 = 'Adapt_Basic';...
            fname_2 = 'AS_Energy_Basic';...
            tab_name_1 = 'Basic'; % Basic Adaptive scheme
    case 2
        fname_1 = 'Adapt_optimal_GSA';...
            fname_2 = 'AS_Energy';...
            tab_name_1 = 'GSA'; % Optimized scheme with Gradient Search Algorithm
    case 3 
        fname_1 = 'Adapt_optimal_GA';...
            fname_2 = 'AS_Energy'; ...
            tab_name_1 = 'GA'; % Optimized scheme with Genetic Algorithm
    case 4
        fname_1 = 'Adapt_optimal_DE'; ...
            fname_2 = 'AS_Energy'; ...
            tab_name_1 = 'DE';% Optimized scheme with Differential Evolution Algorithm
    case 5
       fname_1 = 'Non_Adapt'; ...
           fname_2 = 'AS_Energy'; ...
           tab_name_1 = 'Non_adapt'; % Non controlled scheme
end

% Dividing by days
F_day = 1;
L_day = 365;%F_day;
Range = F_day*24-23:L_day*24; % Amount of hours to perform the test
subrange = 1:24; % It represents the length of a day. It should remain 24
range_1 = [1:7 23:24]; % Low demand hours
range_2 = 8:22; % High demand hours
day = 24;
m = length(Range)/24;
fh = length(value_freq)/(365*24); % Number of frequency measurements per hour

%% Loop for calling the functions

    for i = 1:m % separate each day
        disp('------------------------------------------------------------------\n')
        fprintf('Day %.0f \n SOC_0 = %.2f \n SOC_0_AS = %.2f \n',i,SOC_0,SOC_0_AS)       
        
        % Definition of local variables
        ini = Range((day * (i-1))+1);
        fin = Range(day * i);
        usage_local = EV_usage(ini:fin);
        charging_local = EV_charging(ini:fin); % only applicable to Non-controlled scheme        
        available_local = EV_available(ini:fin);
        Penet_local = Penetration_20(ini:fin);
        freq_local = value_freq((F_day+i-2)*day*fh+1:(F_day+i-1)*day*fh);%(ini*fh:fin*fh);

        
        % Optimize function for Ancillary Services power allocation
        [SOC, Bid, P_up, P_down, ConsumE, fval_up, fval_down] = ... % SOC is estimated SOC
            feval(fname_1,EV_n, EV_P, EV_E, EV_batt, subrange, range_1,...
            range_2, Penet_local,usage_local,available_local, charging_local, SOC_0,i,conv,sch);
        

        [AS_up, AS_down, Ene_AS_down, Ene_AS_up, SOC_AS, P_down_AS, P_up_AS] = ...
            feval(fname_2,SOC_0_AS, Bid, P_up, P_down, EV_batt, EV_n, EV_P,... 
            ConsumE, available_local, freq_local, i, conv);
        
        if sch == 1
            P_down = P_down_AS;
            P_up = P_up_AS;
        end
        
        % Initialize for next day
        SOC_0 = SOC(25) - (SOC(13)-SOC_AS(13))...
            + sum(f_AS_down (4:6) .* P_down (4:6))...
            - sum(f_AS_up (4:6) .* P_up (4:6));
        SOC_0_AS = SOC_AS(25);

        %% Assigning  values to the structure
        matrix_large(i*24-23:i*24,1) = SOC(1:end-1);
        matrix_large(i*24-23:i*24,2) = SOC_AS(1:end-1);
        matrix_large(i*24-23:i*24,3) = Bid;
        matrix_large(i*24-23:i*24,4) = Ene_AS_down;
        matrix_large(i*24-23:i*24,5) = Ene_AS_up;
        
        matrix_short(i*6-5:i*6,1) = AS_down;
        matrix_short(i*6-5:i*6,2) = AS_up;
        matrix_short(i*6-5:i*6,3) = P_down;
        matrix_short(i*6-5:i*6,4) = P_up;
        
        
        if SOC_AS (25) < 0.2*Max_SOC
            disp('SOC lower limit reached')
            break
        elseif SOC_AS (25) > 0.95*Max_SOC
            disp('SOC higher limit reached')
            break     
        end
        
        % Clear temporary variables
        clearvars SOC_non_adapt Charg_non_adapt SOC Bid AS_up AS_down...
            Ene_AS_down Ene_AS_up Cost_non Cost_adapt SOC_AS P_down P_up
        % Clear local variables
        clearvars ini fin usage_local Penet_local spot_local freq_local
        
    end
%% save values to .mat file
% if sch ~= 5
%    if conv == 1
%        tab_name_2 = '_uni_Y';
%    else
%        tab_name_2 = '_bi_Y';
%    end
% else
%     tab_name_2 = [];
% end
    
filename = strcat('Tables\Year_Results\',tab_name_1,tab_name_2,'.mat');    
save(filename,'matrix_large','matrix_short')
toc